class Worker {
  def work() { println 'working...' }
  def analyze() { println 'analyzing...' }
  def takeVacation() { println 'code at the beach...' }
}

class Admin {
  def work() { println 'Admin working...' }
  def report() { println 'prepare report...' }
}

class Manager {
  @Delegate Worker worker = new Worker()
  @Delegate Admin admin = new Admin()

  def takeVacation() { println 'ofcourse I will' }
}

def bob = new Manager()
bob.work()
bob.analyze()
bob.takeVacation()
bob.report()

//Groovy, at compile time, creates a work method in the Manager, and
//route the call to the worker instance.

//Follows the DRY principle - we are not duplicating effort to route in the Manager
//Follows the Open-Closed Principle - if we modify the Worker, we don't have to
//change the Manager to include the new method

//The compiler creates methods only if they already don't exist.
//@Delegate works on order of priority.

//Project Lombok for Java was created due to the influence of Groovy on some
//Java programmers.

